###
# Copyright (c) 2011, Rebecca Bettencourt
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#   * Redistributions of source code must retain the above copyright notice,
#     this list of conditions, and the following disclaimer.
#   * Redistributions in binary form must reproduce the above copyright notice,
#     this list of conditions, and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
#   * Neither the name of the author of this software nor the name of
#     contributors to this software may be used to endorse or promote products
#     derived from this software without specific prior written consent.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

###

import re
import HTMLParser
import htmlentitydefs
import supybot.utils as utils
from supybot.commands import *
import supybot.plugins as plugins
import supybot.ircutils as ircutils
import supybot.callbacks as callbacks

class H1(HTMLParser.HTMLParser):
    entitydefs = htmlentitydefs.entitydefs.copy()
    entitydefs['nbsp'] = ' '
    entitydefs['apos'] = '\''
    def __init__(self):
        self.inH1 = False
        self.h1 = ''
        HTMLParser.HTMLParser.__init__(self)

    def handle_starttag(self, tag, attrs):
        if tag == 'h1':
            self.inH1 = True

    def handle_endtag(self, tag):
        if tag == 'h1':
            self.inH1 = False

    def handle_data(self, data):
        if self.inH1:
            self.h1 += data

    def handle_entityref(self, name):
        if self.inH1:
            if name in self.entitydefs:
                self.h1 += self.entitydefs[name]

class H3(HTMLParser.HTMLParser):
    entitydefs = htmlentitydefs.entitydefs.copy()
    entitydefs['nbsp'] = ' '
    entitydefs['apos'] = '\''
    def __init__(self):
        self.inH3 = False
        self.h3 = ''
        HTMLParser.HTMLParser.__init__(self)

    def handle_starttag(self, tag, attrs):
        if tag == 'h3':
            self.inH3 = True

    def handle_endtag(self, tag):
        if tag == 'h3':
            self.inH3 = False

    def handle_data(self, data):
        if self.inH3 and 'Permanent link to this comic: ' in data:
            self.h3 += data

    def handle_entityref(self, name):
        if self.inH3:
            if name in self.entitydefs:
                self.h3 += self.entitydefs[name]

class XKCD(callbacks.Plugin):
    """This plugin contains commands for referencing a certain webcomic
    about romance, sarcasm, math, and language."""
    threaded = True

    def xkcd(self, irc, msg, args, n):
        """[<comic number>]

        Posts the title and URL of an xkcd comic.
        Defaults to the most recent comic.
        """
        u = 'http://xkcd.com/'
        if n > 0:
            u += str(n)
        c = utils.web.getUrl(u)
        p = H1()
        p.feed(c)
        t = utils.web.htmlToText(p.h1.strip())
        p = H3()
        p.feed(c)
        u = utils.web.htmlToText(p.h3.strip())[30:].strip()
        n = int(u[16:-1])
        irc.reply(format('xkcd #%s: %s - %s', str(n), t, u))
    xkcd = wrap(xkcd, [additional(('int', 'comic number'), 0)])

Class = XKCD

# vim:set shiftwidth=4 softtabstop=4 expandtab textwidth=79:
